﻿using IO.Swagger.Api;
using IO.Swagger.Model;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Forms;


namespace Demo
{
    public partial class Form1 : Form
    {
        private String authenticationToken = "";
        private String dossierToken = "";
        private int dossierId;

        public Form1()
        {
            InitializeComponent();
        }

        private void btngenerateToken_Click(object sender, EventArgs e)
        {
            try
            {
                Credentials credentials = new Credentials(txtUsername.Text, txtPassword.Text);
                AuthenticationApi authentication = new AuthenticationApi();
                AuthenticationToken result = authentication.AuthenticateUser(credentials, txtSoftwareHouseUuid.Text);

                authenticationToken = result.Token;
                
                MessageBox.Show(this, "authentication token generated", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to generate authentication token, \nHTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to generate authentication token, \nreason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnShowToken_Click(object sender, EventArgs e)
        {
            if (!authenticationToken.Equals(""))
                MessageBox.Show(this, authenticationToken, "Authentication Token");
            else
                MessageBox.Show(this, "No authentication token generated yet", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void btnGetDossiers_Click(object sender, EventArgs e)
        {
            try
            {
                DossiersApi dossiersApi = new DossiersApi();
                DossierServiceData[] dossierServiceDataArray = dossiersApi.GetDossiers(authenticationToken).ToArray();
                cmbDossiers.Items.Clear();
                foreach (DossierServiceData d in dossierServiceDataArray)
                {
                    cmbDossiers.Items.Add(new Dossier(d));
                }
                MessageBox.Show(this, "Get dossiers successfull\nSelect a dossier from the combo.", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to fetch dossiers, \nHTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to fetch dossiers, \nreason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnGetDossiertoken_Click(object sender, EventArgs e)
        {
            try
            {
                DossiersApi dossiersApi = new DossiersApi();
                Dossier selectedDossier = (Dossier)cmbDossiers.SelectedItem;
                dossierId = (int)selectedDossier.dossierServiceData.DossierKey.Id;

                DossierTokenResult result = dossiersApi.ConnectDossier(selectedDossier.dossierServiceData.DossierKey.Id, authenticationToken);
                dossierToken = result.Dossiertoken;
                
                MessageBox.Show(this, "dossiertoken generated", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to generate dossiertoken, \nHTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to generate dossiertoken, \nreason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnShowDossierToken_Click(object sender, EventArgs e)
        {
            if (!dossierToken.Equals(""))
                MessageBox.Show(this, dossierToken, "Dossier Token");
            else
                MessageBox.Show(this, "No dossier token generated yet", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void btnGetBookyears_Click(object sender, EventArgs e)
        {
            try
            {
                BookyearsApi bookYearsApi = new BookyearsApi();
                List<BookyearServiceData> bookyearServiceDataArray = bookYearsApi.GetBookyears(dossierId, dossierToken);
                cmbBookyears.Items.Clear();
                foreach (BookyearServiceData b in bookyearServiceDataArray)
                {
                    cmbBookyears.Items.Add(new Bookyear(b));
                }
                MessageBox.Show(this, "Get bookyears successfull\nSelect a bookyear from the combo.", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to fetch bookyears, \nHTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to fetch bookyears, \nreason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); ;
            }
        }

        private void btnGetJournals_Click(object sender, EventArgs e)
        {
            try
            {
                JournalsApi journalsApi = new JournalsApi();
                JournalServiceData[] journalServiceDataArray = journalsApi.GetJournals(dossierId, (int)((Bookyear)cmbBookyears.SelectedItem).bookyearServiceData.BookyearKey.Id, dossierToken).ToArray();
                cmbJournals.Items.Clear();
                foreach (JournalServiceData j in journalServiceDataArray)
                {
                    cmbJournals.Items.Add(new Journal(j));
                }
                MessageBox.Show(this, "Get journals successfull\nSelect a journal from the combo.", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to fetch journals, HTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to fetch journals, reason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnGetRelations_Click(object sender, EventArgs e)
        {
            try
            {
                RelationsApi relationsApi = new RelationsApi();
                RelationServiceData[] relationServiceDataArray = relationsApi.GetRelations(dossierId, dossierToken).ToArray();
                cmbClients.Items.Clear();
                cmbSuppliers.Items.Clear();

                foreach (RelationServiceData r in relationServiceDataArray)
                {
                    if ((Boolean)r.Client)
                    {
                        cmbClients.Items.Add(new Relation(r));
                    }
                    if ((Boolean)r.Supplier)
                    {
                        cmbSuppliers.Items.Add(new Relation(r));
                    }
                }
                MessageBox.Show(this, "Get relations successfull\nSelect a client/supplier from the combo's.", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);

            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to fetch relations, \nHTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to fetch relations, <nreason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void btnGetVatCodes_Click(object sender, EventArgs e)
        {
            try
            {
                VATCodesApi vatCodesApi = new VATCodesApi();
                VatCodeServiceData[] vatCodeArray = vatCodesApi.GetVatCodes(dossierId, dossierToken).ToArray();
                cmbVatCodesBuy.Items.Clear();
                cmbVatCodesSell.Items.Clear();

                foreach (VatCodeServiceData v in vatCodeArray)
                {
                    if (v.Type == VatCode.VT_BUY)
                    {
                        cmbVatCodesBuy.Items.Add(new VatCode(v));
                    }
                    if (v.Type == VatCode.VT_SELL)
                    {
                        cmbVatCodesSell.Items.Add(new VatCode(v));
                    }
                }
                MessageBox.Show(this, "Get vatcodes successfull\nSelect a vatcode from the combo.", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to fetch vat code list, HTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to get vat code list : Reason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private int getLastDocumentSequenceNr()
        {
            Journal journal = (Journal)cmbJournals.SelectedItem;
            return (int)journal.journalServiceData.LastBookedDocumentNr;
        }

        private int findBookyearPeriodNrFromCurrentDate()
        {
            DateTime dtNow = DateTime.Now;
            Bookyear bookyear = (Bookyear)cmbBookyears.SelectedItem;
            BookyearPeriodServiceData[] periods = bookyear.bookyearServiceData.Periods.ToArray();
            foreach (BookyearPeriodServiceData p in periods)
            {
                if (dtNow >= p.StartDate && dtNow <= p.EndDate)
                {
                    return (int)p.BookyearPeriod;
                }
            }
            return -1;
        }

        private void btnInsertInvoice_Click(object sender, EventArgs e)
        {
            try
            {
                InvoicesApi invoicesApi = new InvoicesApi();
                int bookyearPeriod = findBookyearPeriodNrFromCurrentDate();
                if (bookyearPeriod == -1)
                {
                    MessageBox.Show(this, "An appropriate bookyear period for today could\nnot be found in the selected bookyear", "Bookyear Period", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
                RelationIdentificationServiceData relation = ((Relation)cmbClients.SelectedItem).relationServiceData.RelationIdentificationServiceData;
                if (relation.ExternalRelationId == 0 || !relation.ExternalRelationId.HasValue)
                {
                    MessageBox.Show(this, "The external relation id is not set for the selected relation", "Relation", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }

                InvoiceServiceData invoice = new InvoiceServiceData(((Bookyear)cmbBookyears.SelectedItem).bookyearServiceData.BookyearKey, ((Journal)cmbJournals.SelectedItem).journalServiceData.JournalKey,
                    getLastDocumentSequenceNr() + 1, bookyearPeriod, DateTime.Now, DateTime.Now + TimeSpan.FromDays(30), "EUR", 1.0, relation);

                invoice.FinancialDiscount = 0.0;
                invoice.Comment = txtComment.Text;
                invoice.Reference = txtReference.Text;

                // Add invoice line(s)
                invoice.InvoiceLines = new InvoiceLineServiceData[1].ToList<InvoiceLineServiceData>();
                VatCode vatCode = (VatCode)cmbVatCodesSell.SelectedItem;

                invoice.InvoiceLines[0] = new InvoiceLineServiceData(vatCodeKey: vatCode.vatCodeServiceData.Code, bookingAccountNr: vatCode.vatCodeServiceData.DefaultSellBookingAccountNr);
                invoice.InvoiceLines[0].Count = 1.0;
                invoice.InvoiceLines[0].Description = "Test Invoice Demo App";
                invoice.InvoiceLines[0].DiscountPercentage = 0.0;
                invoice.InvoiceLines[0].Unit = "#";
                invoice.InvoiceLines[0].UnitPrice = 250.0;

                invoicesApi.InsertInvoice(invoice, dossierToken, dossierId);
                MessageBox.Show(this, "Invoice inserted successfully", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);

            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to insert invoice, \nHTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to insert invoice : Reason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnInsertSellBooking_Click(object sender, EventArgs e)
        {
            try
            {
                BuySellBookingsApi buySellBookingsApi = new BuySellBookingsApi();
                int bookyearPeriod = findBookyearPeriodNrFromCurrentDate();
                if (bookyearPeriod == -1)
                {
                    MessageBox.Show(this, "An appropriate bookyear period for today could\nnot be found in the selected bookyear", "Bookyear Period", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
                RelationIdentificationServiceData relation = ((Relation)cmbClients.SelectedItem).relationServiceData.RelationIdentificationServiceData;
                if (relation.ExternalRelationId == 0 || !relation.ExternalRelationId.HasValue)
                {
                    MessageBox.Show(this, "The external relation id is not set for the selected relation", "Relation", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
                VatCode vatCode = (VatCode)cmbVatCodesSell.SelectedItem;
                double baseAmount = 242.00;
                double lineVatAmount =  Math.Round(baseAmount * (double)vatCode.vatCodeServiceData.BasePercentage / 100, 2);
                double invoiceTotal = baseAmount + lineVatAmount;

                BuySellBookingServiceData booking = new BuySellBookingServiceData(((Bookyear)cmbBookyears.SelectedItem).bookyearServiceData.BookyearKey, ((Journal)cmbJournals.SelectedItem).journalServiceData.JournalKey,
                    getLastDocumentSequenceNr() + 1, relation, bookyearPeriod, DateTime.Now, DateTime.Now + TimeSpan.FromDays(30), txtComment.Text, txtReference.Text, invoiceTotal, "EUR", 1.0);

                // Add booking line(s)
                booking.BookingLines = new BuySellBookingLineServiceData[1].ToList<BuySellBookingLineServiceData>();
                booking.BookingLines[0] = new BuySellBookingLineServiceData();
                booking.BookingLines[0].AccountKey = vatCode.vatCodeServiceData.DefaultSellBookingAccountNr;

                booking.BookingLines[0].BaseAmount = baseAmount;
                booking.BookingLines[0].Comment = txtComment.Text + " - Line Comment";
                booking.BookingLines[0].VatAmount = lineVatAmount;
                booking.BookingLines[0].VatCodeKey = vatCode.vatCodeServiceData.Code;
                booking.BookingLines[0].VatRecupPercentage = 100.00;

                BuySellBookingAndAttachmentRequest buySellBookingandAttachments = new BuySellBookingAndAttachmentRequest(booking, null);
                buySellBookingsApi.InsertBuySellBooking(buySellBookingandAttachments, dossierToken, dossierId);
                MessageBox.Show(this, "Sell booking inserted successfully", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to insert booking, \nHTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to insert booking : Reason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnSelectAttachement_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                txtBuyAttachement.Text = dlg.FileName;
            }
        }

        private void btnInsertBuyBooking_Click(object sender, EventArgs e)
        {
            try
            {
                BuySellBookingsApi buySellBookingsApi = new BuySellBookingsApi();
                int bookyearPeriod = findBookyearPeriodNrFromCurrentDate();
                if (bookyearPeriod == -1)
                {
                    MessageBox.Show(this, "An appropriate bookyear period for today could\nnot be found in the selected bookyear", "Bookyear Period", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
                RelationIdentificationServiceData relation = ((Relation)cmbSuppliers.SelectedItem).relationServiceData.RelationIdentificationServiceData;
                if (relation.ExternalRelationId == 0 || !relation.ExternalRelationId.HasValue)
                {
                    MessageBox.Show(this, "The external relation id is not set for the selected relation", "Relation", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
                VatCode vatCode = (VatCode)cmbVatCodesBuy.SelectedItem;
                double baseAmount = 242.00;
                double lineVatAmount = Math.Round(baseAmount * (double)vatCode.vatCodeServiceData.BasePercentage / 100, 2);
                double invoiceTotal = baseAmount + lineVatAmount;

                BuySellBookingServiceData booking = new BuySellBookingServiceData(((Bookyear)cmbBookyears.SelectedItem).bookyearServiceData.BookyearKey, ((Journal)cmbJournals.SelectedItem).journalServiceData.JournalKey,
                    getLastDocumentSequenceNr() + 1, relation, bookyearPeriod, DateTime.Now, DateTime.Now + TimeSpan.FromDays(30), txtComment.Text, txtReference.Text, invoiceTotal, "EUR", 1.0);

                // Add booking line(s)
                booking.BookingLines = new BuySellBookingLineServiceData[1].ToList<BuySellBookingLineServiceData>();
                booking.BookingLines[0] = new BuySellBookingLineServiceData();
                booking.BookingLines[0].AccountKey = 600000;

                booking.BookingLines[0].BaseAmount = baseAmount;
                booking.BookingLines[0].Comment = txtComment.Text + " - Line Comment";
                booking.BookingLines[0].VatAmount = lineVatAmount;
                booking.BookingLines[0].VatCodeKey = vatCode.vatCodeServiceData.Code;
                booking.BookingLines[0].VatRecupPercentage = 100.00;


                BuySellBookingAndAttachmentRequest buySellBookingandAttachments = new BuySellBookingAndAttachmentRequest();
                buySellBookingandAttachments.BuySellBookingServiceData = booking;
                if (!String.IsNullOrEmpty(txtBuyAttachement.Text))
                {
                    List<AttachementData> attachements = new List<AttachementData>();
                    AttachementData attachement = new AttachementData();
                    attachement.FileName = Path.GetFileName(txtBuyAttachement.Text);
                    attachement.FileData = File.ReadAllBytes(txtBuyAttachement.Text);
                    attachements.Add(attachement);
                    buySellBookingandAttachments.Attachments = attachements;
                }
                buySellBookingsApi.InsertBuySellBooking(buySellBookingandAttachments, dossierToken, dossierId);
                MessageBox.Show(this, "Buy booking inserted successfully", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to insert booking, \nHTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to insert booking : Reason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void btnGetModfiedBuySellBookings_Click(object sender, EventArgs e)
        {
            try
            {
                BuySellBookingsApi buySellBookingsApi = new BuySellBookingsApi();
                BuySellBookingServiceData[] buySellBookingServiceDataArray = buySellBookingsApi.GetModifiedBuySellBookings(dossierId, dossierToken, (int)((Bookyear)cmbBookyears.SelectedItem).bookyearServiceData.BookyearKey.Id,
                    ((Journal)cmbJournals.SelectedItem).journalServiceData.JournalKey, dateTimePicker1.Value.ToString("yyyy-MM-dd HH:mm:ss.fff")).ToArray();

                Form2 form2 = new Form2();
                String result = "";

                foreach (BuySellBookingServiceData booking in buySellBookingServiceDataArray)
                {
                    result = result + booking.ToJson();
                }
                form2.setResult(result);
                form2.ShowDialog();

            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to fetch Modified Buy/Sell Bookings, HTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to get Modified Buys/Sell Bookings  : Reason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void btnGetProducts_Click(object sender, EventArgs e)
        {
            try
            {
                ProductsApi productsApi = new ProductsApi();
                ProductServiceData[] productServiceDataArray = productsApi.GetProducts(dossierId, dossierToken).ToArray();

                Form2 form2 = new Form2();
                String result = "";

                foreach (ProductServiceData product in productServiceDataArray)
                {
                    result = result + product.ToJson();
                }
                form2.setResult(result);
                form2.ShowDialog();
            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                MessageBox.Show(this, "Failed to fetch products, HTTP response code : " + ex.ErrorCode, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, "Failed to get products : Reason : " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void btnGenerateError_Click(object sender, EventArgs e)
        {
            ErrorsApi errorsApi = new ErrorsApi();
            try
            {
                if (Int32.TryParse(txtErrorNumber.Text, out int errorCode))
                    errorsApi.GenerateSampleErrorWithHttpInfo((errorCode < 0) ? errorCode : -1, cmbLanguage.SelectedIndex + 1);
                else
                    errorsApi.GenerateSampleErrorWithHttpInfo(-1);
            }
            catch (IO.Swagger.Client.ApiException ex)
            {
                if (ex.ErrorCode != 0)
                    MessageBox.Show(this, ex.ErrorContent, "ErrorResult Object sample: HTTP response code: " + ex.ErrorCode, MessageBoxButtons.OK, MessageBoxIcon.Error);
                else
                    MessageBox.Show(this, ex.ErrorContent, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            txtErrorNumber.Text = "-1";
            List<KeyValuePair<string, int>> lstLanguages = new List<KeyValuePair<string, int>>();

            lstLanguages.Add(new KeyValuePair<string, int>("Nederlands", 1));
            lstLanguages.Add(new KeyValuePair<string, int>("Français", 2));
            lstLanguages.Add(new KeyValuePair<string, int>("English", 3));
            lstLanguages.Add(new KeyValuePair<string, int>("German", 4));

            cmbLanguage.DataSource = lstLanguages;
            cmbLanguage.DisplayMember = "Key";
            cmbLanguage.ValueMember = "Value";
        }
    }
}
