# IO.Swagger - the C# library for the Octopus Rest API

API to connect with Octopus

This C# SDK is automatically generated by the [Swagger Codegen](https://github.com/swagger-api/swagger-codegen) project:

- API version: 1.0.0
- SDK version: 1.0.0
- Build package: io.swagger.codegen.v3.generators.dotnet.CSharpClientCodegen

<a name="frameworks-supported"></a>
## Frameworks supported
- .NET 4.0 or later
- Windows Phone 7.1 (Mango)

<a name="dependencies"></a>
## Dependencies
- [RestSharp](https://www.nuget.org/packages/RestSharp) - 105.1.0 or later
- [Json.NET](https://www.nuget.org/packages/Newtonsoft.Json/) - 7.0.0 or later
- [JsonSubTypes](https://www.nuget.org/packages/JsonSubTypes/) - 1.2.0 or later

The DLLs included in the package may not be the latest version. We recommend using [NuGet](https://docs.nuget.org/consume/installing-nuget) to obtain the latest version of the packages:
```
Install-Package RestSharp
Install-Package Newtonsoft.Json
Install-Package JsonSubTypes
```

NOTE: RestSharp versions greater than 105.1.0 have a bug which causes file uploads to fail. See [RestSharp#742](https://github.com/restsharp/RestSharp/issues/742)

<a name="installation"></a>
## Installation
Run the following command to generate the DLL
- [Mac/Linux] `/bin/sh build.sh`
- [Windows] `build.bat`

Then include the DLL (under the `bin` folder) in the C# project, and use the namespaces:
```csharp
using IO.Swagger.Api;
using IO.Swagger.Client;
using IO.Swagger.Model;
```
<a name="packaging"></a>
## Packaging

A `.nuspec` is included with the project. You can follow the Nuget quickstart to [create](https://docs.microsoft.com/en-us/nuget/quickstart/create-and-publish-a-package#create-the-package) and [publish](https://docs.microsoft.com/en-us/nuget/quickstart/create-and-publish-a-package#publish-the-package) packages.

This `.nuspec` uses placeholders from the `.csproj`, so build the `.csproj` directly:

```
nuget pack -Build -OutputDirectory out IO.Swagger.csproj
```

Then, publish to a [local feed](https://docs.microsoft.com/en-us/nuget/hosting-packages/local-feeds) or [other host](https://docs.microsoft.com/en-us/nuget/hosting-packages/overview) and consume the new package via Nuget as usual.

<a name="getting-started"></a>
## Getting Started

```csharp
using System;
using System.Diagnostics;
using IO.Swagger.Api;
using IO.Swagger.Client;
using IO.Swagger.Model;

namespace Example
{
    public class Example
    {
        public void main()
        {
            var apiInstance = new AccountsApi();
            var dossierId = 56;  // int? | Dossier ID
            var bookYear = 56;  // int? | Bookyear ID
            var dossierToken = dossierToken_example;  // string | Dossier Token
            var modifiedTimeStamp = modifiedTimeStamp_example;  // string | Modified Timestamp (optional) 

            try
            {
                // Get Accounts
                InlineResponse200 result = apiInstance.GetAccounts(dossierId, bookYear, dossierToken, modifiedTimeStamp);
                Debug.WriteLine(result);
            }
            catch (Exception e)
            {
                Debug.Print("Exception when calling AccountsApi.GetAccounts: " + e.Message );
            }
        }
    }
}
```

<a name="documentation-for-api-endpoints"></a>
## Documentation for API Endpoints

All URIs are relative to */octopus-rest-api/v1*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*AccountsApi* | [**GetAccounts**](docs/AccountsApi.md#getaccounts) | **GET** /dossiers/{dossierId}/BookYears/{bookYear}/Accounts | Get Accounts
*AccountsApi* | [**InsertUpdateAccount**](docs/AccountsApi.md#insertupdateaccount) | **PUT** /dossiers/{dossierId}/BookYears/{bookYear}/Accounts/{accountId} | Modify or Create Account
*AuthenticationApi* | [**AuthenticateUser**](docs/AuthenticationApi.md#authenticateuser) | **POST** /authentication | Authenticate
*BookYearsApi* | [**GetBookYears**](docs/BookYearsApi.md#getbookyears) | **GET** /dossiers/{dossierId}/bookyears | Get BookYears
*BuySellBookingsApi* | [**CreateBuySellBooking**](docs/BuySellBookingsApi.md#createbuysellbooking) | **POST** /dossiers/{dossierId}/buysellbookings | Create Booking
*BuySellBookingsApi* | [**GetBuySellBooking**](docs/BuySellBookingsApi.md#getbuysellbooking) | **GET** /dossiers/{dossierId}/buysellbookings | Get Buy Sell Booking
*BuySellBookingsApi* | [**GetModifiedBuySellBookings**](docs/BuySellBookingsApi.md#getmodifiedbuysellbookings) | **GET** /dossiers/{dossierId}/buysellbookings/modified | Get modified Buy/Sell Bookings
*BuySellBookingsApi* | [**UpdateBuySellBooking**](docs/BuySellBookingsApi.md#updatebuysellbooking) | **PUT** /dossiers/{dossierId}/buysellbookings | Update Booking
*CodaApi* | [**ImportCoda**](docs/CodaApi.md#importcoda) | **POST** /dossiers/{dossierId}/coda | Import coda
*CostcentresApi* | [**CreateCostCentre**](docs/CostcentresApi.md#createcostcentre) | **POST** /dossiers/{dossierId}/costcentres | Create Costcentre
*CostcentresApi* | [**GetCostcentres**](docs/CostcentresApi.md#getcostcentres) | **GET** /dossiers/{dossierId}/costcentres | Get Costcentres
*CostcentresApi* | [**UpdateCostCentre**](docs/CostcentresApi.md#updatecostcentre) | **PUT** /dossiers/{dossierId}/costcentres/{costcentreId} | Update Costcentre
*DeliveryNotesApi* | [**CreateDeliveryNote**](docs/DeliveryNotesApi.md#createdeliverynote) | **POST** /dossiers/{dossierId}/deliverynotes | Create Delivery Note
*DeliveryNotesApi* | [**ExportDeliveryNote**](docs/DeliveryNotesApi.md#exportdeliverynote) | **GET** /dossiers/{dossierId}/deliverynotes/export | Export Delivery Note
*DeliveryNotesApi* | [**GetDeliveryNote**](docs/DeliveryNotesApi.md#getdeliverynote) | **GET** /dossiers/{dossierId}/deliverynotes | Get Delivery Note
*DeliveryNotesApi* | [**GetModifiedDeliveryNotes**](docs/DeliveryNotesApi.md#getmodifieddeliverynotes) | **GET** /dossiers/{dossierId}/deliverynotes/modified | Get modified Delivery Notes
*DeliveryNotesApi* | [**UpdateDeliveryNote**](docs/DeliveryNotesApi.md#updatedeliverynote) | **PUT** /dossiers/{dossierId}/deliverynotes | Update Delivery Note
*DossiersApi* | [**ConnectDossier**](docs/DossiersApi.md#connectdossier) | **POST** /dossiers | Get Dossiertoken
*DossiersApi* | [**GetDossiers**](docs/DossiersApi.md#getdossiers) | **GET** /dossiers | Get Dossiers
*FinancialDiversBookingsApi* | [**CreateFinancialDiversBooking**](docs/FinancialDiversBookingsApi.md#createfinancialdiversbooking) | **POST** /dossiers/{dossierId}/financialdiversbookings | Create Booking
*FinancialDiversBookingsApi* | [**GetFinancialDiversBooking**](docs/FinancialDiversBookingsApi.md#getfinancialdiversbooking) | **GET** /dossiers/{dossierId}/financialdiversbookings | Get Financial Divers Booking
*FinancialDiversBookingsApi* | [**GetFinancialDiversModifiedBookings**](docs/FinancialDiversBookingsApi.md#getfinancialdiversmodifiedbookings) | **GET** /dossiers/{dossierId}/financialdiversbookings/modified | Get modified Financial Divers Bookings
*FinancialDiversBookingsApi* | [**UpdateFinancialDiversBooking**](docs/FinancialDiversBookingsApi.md#updatefinancialdiversbooking) | **PUT** /dossiers/{dossierId}/financialdiversbookings | Update Bookings
*InvoicesApi* | [**CreateInvoice**](docs/InvoicesApi.md#createinvoice) | **POST** /dossiers/{dossierId}/invoices | Create invoice
*InvoicesApi* | [**ExportInvoice**](docs/InvoicesApi.md#exportinvoice) | **GET** /dossiers/{dossierId}/invoices/export | Export Invoice
*InvoicesApi* | [**GetInvoice**](docs/InvoicesApi.md#getinvoice) | **GET** /dossiers/{dossierId}/invoices | Get Invoice
*InvoicesApi* | [**GetModifiedInvoices**](docs/InvoicesApi.md#getmodifiedinvoices) | **GET** /dossiers/{dossierId}/invoices/modified | Get modified invoices
*InvoicesApi* | [**UpdateInvoice**](docs/InvoicesApi.md#updateinvoice) | **PUT** /dossiers/{dossierId}/invoices | Update invoice
*JournalsApi* | [**GetFinancialJournalBalance**](docs/JournalsApi.md#getfinancialjournalbalance) | **GET** /dossiers/{dossierId}/BookYears/{bookyearId}/Journals/{journalKey} | Get Financial Journal Balance
*JournalsApi* | [**GetJournals**](docs/JournalsApi.md#getjournals) | **GET** /dossiers/{dossierId}/BookYears/{bookyearId}/Journals | Get Journals
*ProductgroupsApi* | [**GetProductgroups**](docs/ProductgroupsApi.md#getproductgroups) | **GET** /dossiers/{dossierId}/productgroups | Get Productgroups
*ProductsApi* | [**CreateProduct**](docs/ProductsApi.md#createproduct) | **POST** /dossiers/{dossierId}/products | Create Product
*ProductsApi* | [**GetProducts**](docs/ProductsApi.md#getproducts) | **GET** /dossiers/{dossierId}/products | Get Products
*ProductsApi* | [**UpdateProduct**](docs/ProductsApi.md#updateproduct) | **PUT** /dossiers/{dossierId}/products/{productId} | Modify Product
*RappelsApi* | [**ExportRappel**](docs/RappelsApi.md#exportrappel) | **GET** /dossiers/{dossierId}/rappels/export | Export Rappel
*RappelsApi* | [**GetRappelInformation**](docs/RappelsApi.md#getrappelinformation) | **GET** /dossiers/{dossierId}/rappels | Get Rappel information
*RelationsApi* | [**CreateRelation**](docs/RelationsApi.md#createrelation) | **POST** /dossiers/{dossierId}/relations | Create Relation
*RelationsApi* | [**GetRelations**](docs/RelationsApi.md#getrelations) | **GET** /dossiers/{dossierId}/relations | Get Relations
*RelationsApi* | [**UpdateRelation**](docs/RelationsApi.md#updaterelation) | **PUT** /dossiers/{dossierId}/relations/{relationId} | Modify Relation
*ReportsApi* | [**GetAccountOpenReport**](docs/ReportsApi.md#getaccountopenreport) | **POST** /dossiers/{dossierId}/reports/accounts/open | Get open account report
*ReportsApi* | [**GetAccountsHistoryReport**](docs/ReportsApi.md#getaccountshistoryreport) | **POST** /dossiers/{dossierId}/reports/accounts/history | Get history account report
*ReportsApi* | [**GetClientHistoryReport**](docs/ReportsApi.md#getclienthistoryreport) | **POST** /dossiers/{dossierId}/reports/clients/history | Get history client report
*ReportsApi* | [**GetClientOpenReport**](docs/ReportsApi.md#getclientopenreport) | **POST** /dossiers/{dossierId}/reports/clients/open | Get open client report
*ReportsApi* | [**GetCostCentreHistoryReport**](docs/ReportsApi.md#getcostcentrehistoryreport) | **POST** /dossiers/{dossierId}/reports/costcentres/history | Get history costcentre report
*ReportsApi* | [**GetSupplierHistoryReport**](docs/ReportsApi.md#getsupplierhistoryreport) | **POST** /dossiers/{dossierId}/reports/suppliers/history | Get history supplier report
*ReportsApi* | [**GetSupplierOpenReport**](docs/ReportsApi.md#getsupplieropenreport) | **POST** /dossiers/{dossierId}/reports/suppliers/open | Get open supplier report
*VATCodesApi* | [**GetVatCodes**](docs/VATCodesApi.md#getvatcodes) | **GET** /dossiers/{dossierId}/vatcodes | Get VAT Codes

<a name="documentation-for-models"></a>
## Documentation for Models

 - [Model.AccountKey](docs/AccountKey.md)
 - [Model.AccountServiceData](docs/AccountServiceData.md)
 - [Model.AttachementData](docs/AttachementData.md)
 - [Model.BookyearKey](docs/BookyearKey.md)
 - [Model.BookyearPeriodServiceData](docs/BookyearPeriodServiceData.md)
 - [Model.BookyearServiceData](docs/BookyearServiceData.md)
 - [Model.BuySellBookingLineServiceData](docs/BuySellBookingLineServiceData.md)
 - [Model.BuySellBookingServiceData](docs/BuySellBookingServiceData.md)
 - [Model.BuySellBookingandAttachmentsRequest](docs/BuySellBookingandAttachmentsRequest.md)
 - [Model.CodaImportItemReportServiceData](docs/CodaImportItemReportServiceData.md)
 - [Model.CodaImportReportServiceData](docs/CodaImportReportServiceData.md)
 - [Model.CodaLineServiceData](docs/CodaLineServiceData.md)
 - [Model.CodaVersionData](docs/CodaVersionData.md)
 - [Model.CostCentreKey](docs/CostCentreKey.md)
 - [Model.CostCentreServiceData](docs/CostCentreServiceData.md)
 - [Model.Credentials](docs/Credentials.md)
 - [Model.CustomFieldKey](docs/CustomFieldKey.md)
 - [Model.CustomFieldValueServiceData](docs/CustomFieldValueServiceData.md)
 - [Model.DeliveryNoteLineServiceData](docs/DeliveryNoteLineServiceData.md)
 - [Model.DeliveryNoteServiceData](docs/DeliveryNoteServiceData.md)
 - [Model.DossierKey](docs/DossierKey.md)
 - [Model.DossierServiceData](docs/DossierServiceData.md)
 - [Model.ExportedDeliveryNoteResult](docs/ExportedDeliveryNoteResult.md)
 - [Model.ExportedInvoiceResult](docs/ExportedInvoiceResult.md)
 - [Model.ExportedRappelResult](docs/ExportedRappelResult.md)
 - [Model.FinancialDiversBookingLineServiceData](docs/FinancialDiversBookingLineServiceData.md)
 - [Model.FinancialDiversBookingServiceData](docs/FinancialDiversBookingServiceData.md)
 - [Model.FinancialDiversBookingandAttachmentsRequest](docs/FinancialDiversBookingandAttachmentsRequest.md)
 - [Model.HistoryAccountQueryData](docs/HistoryAccountQueryData.md)
 - [Model.HistoryCostCentreData](docs/HistoryCostCentreData.md)
 - [Model.HistoryCostCentreQueryData](docs/HistoryCostCentreQueryData.md)
 - [Model.HistoryData](docs/HistoryData.md)
 - [Model.HistoryRelationQueryData](docs/HistoryRelationQueryData.md)
 - [Model.InlineResponse200](docs/InlineResponse200.md)
 - [Model.InlineResponse2001](docs/InlineResponse2001.md)
 - [Model.InvoiceLineServiceData](docs/InvoiceLineServiceData.md)
 - [Model.InvoiceServiceData](docs/InvoiceServiceData.md)
 - [Model.JournalBalance](docs/JournalBalance.md)
 - [Model.JournalServiceData](docs/JournalServiceData.md)
 - [Model.OneOfinlineResponse200](docs/OneOfinlineResponse200.md)
 - [Model.OneOfinlineResponse2001](docs/OneOfinlineResponse2001.md)
 - [Model.OpenAccountQueryData](docs/OpenAccountQueryData.md)
 - [Model.OpenData](docs/OpenData.md)
 - [Model.OpenRelationQueryData](docs/OpenRelationQueryData.md)
 - [Model.ProductGroupKey](docs/ProductGroupKey.md)
 - [Model.ProductGroupServiceData](docs/ProductGroupServiceData.md)
 - [Model.ProductKey](docs/ProductKey.md)
 - [Model.ProductServiceData](docs/ProductServiceData.md)
 - [Model.RappelInfoInvoiceServiceData](docs/RappelInfoInvoiceServiceData.md)
 - [Model.RappelInvoiceServiceData](docs/RappelInvoiceServiceData.md)
 - [Model.RappelServiceData](docs/RappelServiceData.md)
 - [Model.RelationIdentificationServiceData](docs/RelationIdentificationServiceData.md)
 - [Model.RelationKey](docs/RelationKey.md)
 - [Model.RelationServiceData](docs/RelationServiceData.md)
 - [Model.StringServiceData](docs/StringServiceData.md)
 - [Model.SyncAccounts](docs/SyncAccounts.md)
 - [Model.SyncRelations](docs/SyncRelations.md)
 - [Model.VatCodeServiceData](docs/VatCodeServiceData.md)

<a name="documentation-for-authorization"></a>
## Documentation for Authorization

All endpoints do not require authorization.
