/* 
 * Octopus Rest API
 *
 * API to connect with Octopus
 *
 * OpenAPI spec version: 1.0
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using RestSharp;
using IO.Swagger.Client;
using IO.Swagger.Model;

namespace IO.Swagger.Api
{
    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
        public interface IAccountsApi : IApiAccessor
    {
        #region Synchronous Operations
        /// <summary>
        /// Get Accounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token and return Accounts.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="bookyearId">Bookyear ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>List&lt;AccountServiceData&gt;</returns>
        List<AccountServiceData> GetAccounts (int? dossierId, int? bookyearId, string dossierToken);

        /// <summary>
        /// Get Accounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token and return Accounts.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="bookyearId">Bookyear ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>ApiResponse of List&lt;AccountServiceData&gt;</returns>
        ApiResponse<List<AccountServiceData>> GetAccountsWithHttpInfo (int? dossierId, int? bookyearId, string dossierToken);
        /// <summary>
        /// Get Modified Account Amounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token, modified timestamp, and return modified/deleted Account Amounts.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp (optional)</param>
        /// <returns>SyncAccountAmounts</returns>
        SyncAccountAmounts GetModifiedAccountAmounts (int? dossierId, string dossierToken, string modifiedTimeStamp = null);

        /// <summary>
        /// Get Modified Account Amounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token, modified timestamp, and return modified/deleted Account Amounts.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp (optional)</param>
        /// <returns>ApiResponse of SyncAccountAmounts</returns>
        ApiResponse<SyncAccountAmounts> GetModifiedAccountAmountsWithHttpInfo (int? dossierId, string dossierToken, string modifiedTimeStamp = null);
        /// <summary>
        /// Get Modified Accounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token, modified timestamp, and return modified/deleted Accounts.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>SyncAccounts</returns>
        SyncAccounts GetModifiedAccounts (int? dossierId, string modifiedTimeStamp, string dossierToken);

        /// <summary>
        /// Get Modified Accounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token, modified timestamp, and return modified/deleted Accounts.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>ApiResponse of SyncAccounts</returns>
        ApiResponse<SyncAccounts> GetModifiedAccountsWithHttpInfo (int? dossierId, string modifiedTimeStamp, string dossierToken);
        /// <summary>
        /// Modify or Create Account
        /// </summary>
        /// <remarks>
        /// Accept dossier token and modified or new account.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Account</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <returns></returns>
        void InsertUpdateAccount (AccountServiceData body, string dossierToken, int? dossierId);

        /// <summary>
        /// Modify or Create Account
        /// </summary>
        /// <remarks>
        /// Accept dossier token and modified or new account.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Account</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <returns>ApiResponse of Object(void)</returns>
        ApiResponse<Object> InsertUpdateAccountWithHttpInfo (AccountServiceData body, string dossierToken, int? dossierId);
        #endregion Synchronous Operations
        #region Asynchronous Operations
        /// <summary>
        /// Get Accounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token and return Accounts.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="bookyearId">Bookyear ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>Task of List&lt;AccountServiceData&gt;</returns>
        System.Threading.Tasks.Task<List<AccountServiceData>> GetAccountsAsync (int? dossierId, int? bookyearId, string dossierToken);

        /// <summary>
        /// Get Accounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token and return Accounts.
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="bookyearId">Bookyear ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>Task of ApiResponse (List&lt;AccountServiceData&gt;)</returns>
        System.Threading.Tasks.Task<ApiResponse<List<AccountServiceData>>> GetAccountsAsyncWithHttpInfo (int? dossierId, int? bookyearId, string dossierToken);
        /// <summary>
        /// Get Modified Account Amounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token, modified timestamp, and return modified/deleted Account Amounts.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp (optional)</param>
        /// <returns>Task of SyncAccountAmounts</returns>
        System.Threading.Tasks.Task<SyncAccountAmounts> GetModifiedAccountAmountsAsync (int? dossierId, string dossierToken, string modifiedTimeStamp = null);

        /// <summary>
        /// Get Modified Account Amounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token, modified timestamp, and return modified/deleted Account Amounts.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp (optional)</param>
        /// <returns>Task of ApiResponse (SyncAccountAmounts)</returns>
        System.Threading.Tasks.Task<ApiResponse<SyncAccountAmounts>> GetModifiedAccountAmountsAsyncWithHttpInfo (int? dossierId, string dossierToken, string modifiedTimeStamp = null);
        /// <summary>
        /// Get Modified Accounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token, modified timestamp, and return modified/deleted Accounts.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>Task of SyncAccounts</returns>
        System.Threading.Tasks.Task<SyncAccounts> GetModifiedAccountsAsync (int? dossierId, string modifiedTimeStamp, string dossierToken);

        /// <summary>
        /// Get Modified Accounts
        /// </summary>
        /// <remarks>
        /// Accept dossier token, modified timestamp, and return modified/deleted Accounts.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>Task of ApiResponse (SyncAccounts)</returns>
        System.Threading.Tasks.Task<ApiResponse<SyncAccounts>> GetModifiedAccountsAsyncWithHttpInfo (int? dossierId, string modifiedTimeStamp, string dossierToken);
        /// <summary>
        /// Modify or Create Account
        /// </summary>
        /// <remarks>
        /// Accept dossier token and modified or new account.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Account</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <returns>Task of void</returns>
        System.Threading.Tasks.Task InsertUpdateAccountAsync (AccountServiceData body, string dossierToken, int? dossierId);

        /// <summary>
        /// Modify or Create Account
        /// </summary>
        /// <remarks>
        /// Accept dossier token and modified or new account.  
        /// </remarks>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Account</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <returns>Task of ApiResponse</returns>
        System.Threading.Tasks.Task<ApiResponse<Object>> InsertUpdateAccountAsyncWithHttpInfo (AccountServiceData body, string dossierToken, int? dossierId);
        #endregion Asynchronous Operations
    }

    /// <summary>
    /// Represents a collection of functions to interact with the API endpoints
    /// </summary>
        public partial class AccountsApi : IAccountsApi
    {
        private IO.Swagger.Client.ExceptionFactory _exceptionFactory = (name, response) => null;

        /// <summary>
        /// Initializes a new instance of the <see cref="AccountsApi"/> class.
        /// </summary>
        /// <returns></returns>
        public AccountsApi(String basePath)
        {
            this.Configuration = new IO.Swagger.Client.Configuration { BasePath = basePath };

            ExceptionFactory = IO.Swagger.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AccountsApi"/> class
        /// </summary>
        /// <returns></returns>
        public AccountsApi()
        {
            this.Configuration = IO.Swagger.Client.Configuration.Default;

            ExceptionFactory = IO.Swagger.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="AccountsApi"/> class
        /// using Configuration object
        /// </summary>
        /// <param name="configuration">An instance of Configuration</param>
        /// <returns></returns>
        public AccountsApi(IO.Swagger.Client.Configuration configuration = null)
        {
            if (configuration == null) // use the default one in Configuration
                this.Configuration = IO.Swagger.Client.Configuration.Default;
            else
                this.Configuration = configuration;

            ExceptionFactory = IO.Swagger.Client.Configuration.DefaultExceptionFactory;
        }

        /// <summary>
        /// Gets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        public String GetBasePath()
        {
            return this.Configuration.ApiClient.RestClient.BaseUrl.ToString();
        }

        /// <summary>
        /// Sets the base path of the API client.
        /// </summary>
        /// <value>The base path</value>
        [Obsolete("SetBasePath is deprecated, please do 'Configuration.ApiClient = new ApiClient(\"http://new-path\")' instead.")]
        public void SetBasePath(String basePath)
        {
            // do nothing
        }

        /// <summary>
        /// Gets or sets the configuration object
        /// </summary>
        /// <value>An instance of the Configuration</value>
        public IO.Swagger.Client.Configuration Configuration {get; set;}

        /// <summary>
        /// Provides a factory method hook for the creation of exceptions.
        /// </summary>
        public IO.Swagger.Client.ExceptionFactory ExceptionFactory
        {
            get
            {
                if (_exceptionFactory != null && _exceptionFactory.GetInvocationList().Length > 1)
                {
                    throw new InvalidOperationException("Multicast delegate for ExceptionFactory is unsupported.");
                }
                return _exceptionFactory;
            }
            set { _exceptionFactory = value; }
        }

        /// <summary>
        /// Gets the default header.
        /// </summary>
        /// <returns>Dictionary of HTTP header</returns>
        [Obsolete("DefaultHeader is deprecated, please use Configuration.DefaultHeader instead.")]
        public IDictionary<String, String> DefaultHeader()
        {
            return new ReadOnlyDictionary<string, string>(this.Configuration.DefaultHeader);
        }

        /// <summary>
        /// Add default header.
        /// </summary>
        /// <param name="key">Header field name.</param>
        /// <param name="value">Header field value.</param>
        /// <returns></returns>
        [Obsolete("AddDefaultHeader is deprecated, please use Configuration.AddDefaultHeader instead.")]
        public void AddDefaultHeader(string key, string value)
        {
            this.Configuration.AddDefaultHeader(key, value);
        }

        /// <summary>
        /// Get Accounts Accept dossier token and return Accounts.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="bookyearId">Bookyear ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>List&lt;AccountServiceData&gt;</returns>
        public List<AccountServiceData> GetAccounts (int? dossierId, int? bookyearId, string dossierToken)
        {
             ApiResponse<List<AccountServiceData>> localVarResponse = GetAccountsWithHttpInfo(dossierId, bookyearId, dossierToken);
             return localVarResponse.Data;
        }

        /// <summary>
        /// Get Accounts Accept dossier token and return Accounts.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="bookyearId">Bookyear ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>ApiResponse of List&lt;AccountServiceData&gt;</returns>
        public ApiResponse< List<AccountServiceData> > GetAccountsWithHttpInfo (int? dossierId, int? bookyearId, string dossierToken)
        {
            // verify the required parameter 'dossierId' is set
            if (dossierId == null)
                throw new ApiException(400, "Missing required parameter 'dossierId' when calling AccountsApi->GetAccounts");
            // verify the required parameter 'bookyearId' is set
            if (bookyearId == null)
                throw new ApiException(400, "Missing required parameter 'bookyearId' when calling AccountsApi->GetAccounts");
            // verify the required parameter 'dossierToken' is set
            if (dossierToken == null)
                throw new ApiException(400, "Missing required parameter 'dossierToken' when calling AccountsApi->GetAccounts");

            var localVarPath = "/dossiers/{dossierId}/accounts";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (dossierId != null) localVarPathParams.Add("dossierId", this.Configuration.ApiClient.ParameterToString(dossierId)); // path parameter
            if (bookyearId != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "bookyearId", bookyearId)); // query parameter
            if (dossierToken != null) localVarHeaderParams.Add("dossierToken", this.Configuration.ApiClient.ParameterToString(dossierToken)); // header parameter

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) this.Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetAccounts", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<List<AccountServiceData>>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (List<AccountServiceData>) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(List<AccountServiceData>)));
        }

        /// <summary>
        /// Get Accounts Accept dossier token and return Accounts.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="bookyearId">Bookyear ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>Task of List&lt;AccountServiceData&gt;</returns>
        public async System.Threading.Tasks.Task<List<AccountServiceData>> GetAccountsAsync (int? dossierId, int? bookyearId, string dossierToken)
        {
             ApiResponse<List<AccountServiceData>> localVarResponse = await GetAccountsAsyncWithHttpInfo(dossierId, bookyearId, dossierToken);
             return localVarResponse.Data;

        }

        /// <summary>
        /// Get Accounts Accept dossier token and return Accounts.
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="bookyearId">Bookyear ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>Task of ApiResponse (List&lt;AccountServiceData&gt;)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<List<AccountServiceData>>> GetAccountsAsyncWithHttpInfo (int? dossierId, int? bookyearId, string dossierToken)
        {
            // verify the required parameter 'dossierId' is set
            if (dossierId == null)
                throw new ApiException(400, "Missing required parameter 'dossierId' when calling AccountsApi->GetAccounts");
            // verify the required parameter 'bookyearId' is set
            if (bookyearId == null)
                throw new ApiException(400, "Missing required parameter 'bookyearId' when calling AccountsApi->GetAccounts");
            // verify the required parameter 'dossierToken' is set
            if (dossierToken == null)
                throw new ApiException(400, "Missing required parameter 'dossierToken' when calling AccountsApi->GetAccounts");

            var localVarPath = "/dossiers/{dossierId}/accounts";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (dossierId != null) localVarPathParams.Add("dossierId", this.Configuration.ApiClient.ParameterToString(dossierId)); // path parameter
            if (bookyearId != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "bookyearId", bookyearId)); // query parameter
            if (dossierToken != null) localVarHeaderParams.Add("dossierToken", this.Configuration.ApiClient.ParameterToString(dossierToken)); // header parameter

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await this.Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetAccounts", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<List<AccountServiceData>>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (List<AccountServiceData>) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(List<AccountServiceData>)));
        }

        /// <summary>
        /// Get Modified Account Amounts Accept dossier token, modified timestamp, and return modified/deleted Account Amounts.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp (optional)</param>
        /// <returns>SyncAccountAmounts</returns>
        public SyncAccountAmounts GetModifiedAccountAmounts (int? dossierId, string dossierToken, string modifiedTimeStamp = null)
        {
             ApiResponse<SyncAccountAmounts> localVarResponse = GetModifiedAccountAmountsWithHttpInfo(dossierId, dossierToken, modifiedTimeStamp);
             return localVarResponse.Data;
        }

        /// <summary>
        /// Get Modified Account Amounts Accept dossier token, modified timestamp, and return modified/deleted Account Amounts.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp (optional)</param>
        /// <returns>ApiResponse of SyncAccountAmounts</returns>
        public ApiResponse< SyncAccountAmounts > GetModifiedAccountAmountsWithHttpInfo (int? dossierId, string dossierToken, string modifiedTimeStamp = null)
        {
            // verify the required parameter 'dossierId' is set
            if (dossierId == null)
                throw new ApiException(400, "Missing required parameter 'dossierId' when calling AccountsApi->GetModifiedAccountAmounts");
            // verify the required parameter 'dossierToken' is set
            if (dossierToken == null)
                throw new ApiException(400, "Missing required parameter 'dossierToken' when calling AccountsApi->GetModifiedAccountAmounts");

            var localVarPath = "/dossiers/{dossierId}/accounts/amounts/modified";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (dossierId != null) localVarPathParams.Add("dossierId", this.Configuration.ApiClient.ParameterToString(dossierId)); // path parameter
            if (modifiedTimeStamp != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "modifiedTimeStamp", modifiedTimeStamp)); // query parameter
            if (dossierToken != null) localVarHeaderParams.Add("dossierToken", this.Configuration.ApiClient.ParameterToString(dossierToken)); // header parameter

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) this.Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetModifiedAccountAmounts", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<SyncAccountAmounts>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (SyncAccountAmounts) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(SyncAccountAmounts)));
        }

        /// <summary>
        /// Get Modified Account Amounts Accept dossier token, modified timestamp, and return modified/deleted Account Amounts.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp (optional)</param>
        /// <returns>Task of SyncAccountAmounts</returns>
        public async System.Threading.Tasks.Task<SyncAccountAmounts> GetModifiedAccountAmountsAsync (int? dossierId, string dossierToken, string modifiedTimeStamp = null)
        {
             ApiResponse<SyncAccountAmounts> localVarResponse = await GetModifiedAccountAmountsAsyncWithHttpInfo(dossierId, dossierToken, modifiedTimeStamp);
             return localVarResponse.Data;

        }

        /// <summary>
        /// Get Modified Account Amounts Accept dossier token, modified timestamp, and return modified/deleted Account Amounts.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp (optional)</param>
        /// <returns>Task of ApiResponse (SyncAccountAmounts)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<SyncAccountAmounts>> GetModifiedAccountAmountsAsyncWithHttpInfo (int? dossierId, string dossierToken, string modifiedTimeStamp = null)
        {
            // verify the required parameter 'dossierId' is set
            if (dossierId == null)
                throw new ApiException(400, "Missing required parameter 'dossierId' when calling AccountsApi->GetModifiedAccountAmounts");
            // verify the required parameter 'dossierToken' is set
            if (dossierToken == null)
                throw new ApiException(400, "Missing required parameter 'dossierToken' when calling AccountsApi->GetModifiedAccountAmounts");

            var localVarPath = "/dossiers/{dossierId}/accounts/amounts/modified";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (dossierId != null) localVarPathParams.Add("dossierId", this.Configuration.ApiClient.ParameterToString(dossierId)); // path parameter
            if (modifiedTimeStamp != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "modifiedTimeStamp", modifiedTimeStamp)); // query parameter
            if (dossierToken != null) localVarHeaderParams.Add("dossierToken", this.Configuration.ApiClient.ParameterToString(dossierToken)); // header parameter

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await this.Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetModifiedAccountAmounts", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<SyncAccountAmounts>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (SyncAccountAmounts) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(SyncAccountAmounts)));
        }

        /// <summary>
        /// Get Modified Accounts Accept dossier token, modified timestamp, and return modified/deleted Accounts.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>SyncAccounts</returns>
        public SyncAccounts GetModifiedAccounts (int? dossierId, string modifiedTimeStamp, string dossierToken)
        {
             ApiResponse<SyncAccounts> localVarResponse = GetModifiedAccountsWithHttpInfo(dossierId, modifiedTimeStamp, dossierToken);
             return localVarResponse.Data;
        }

        /// <summary>
        /// Get Modified Accounts Accept dossier token, modified timestamp, and return modified/deleted Accounts.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>ApiResponse of SyncAccounts</returns>
        public ApiResponse< SyncAccounts > GetModifiedAccountsWithHttpInfo (int? dossierId, string modifiedTimeStamp, string dossierToken)
        {
            // verify the required parameter 'dossierId' is set
            if (dossierId == null)
                throw new ApiException(400, "Missing required parameter 'dossierId' when calling AccountsApi->GetModifiedAccounts");
            // verify the required parameter 'modifiedTimeStamp' is set
            if (modifiedTimeStamp == null)
                throw new ApiException(400, "Missing required parameter 'modifiedTimeStamp' when calling AccountsApi->GetModifiedAccounts");
            // verify the required parameter 'dossierToken' is set
            if (dossierToken == null)
                throw new ApiException(400, "Missing required parameter 'dossierToken' when calling AccountsApi->GetModifiedAccounts");

            var localVarPath = "/dossiers/{dossierId}/accounts/modified";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (dossierId != null) localVarPathParams.Add("dossierId", this.Configuration.ApiClient.ParameterToString(dossierId)); // path parameter
            if (modifiedTimeStamp != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "modifiedTimeStamp", modifiedTimeStamp)); // query parameter
            if (dossierToken != null) localVarHeaderParams.Add("dossierToken", this.Configuration.ApiClient.ParameterToString(dossierToken)); // header parameter

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) this.Configuration.ApiClient.CallApi(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetModifiedAccounts", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<SyncAccounts>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (SyncAccounts) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(SyncAccounts)));
        }

        /// <summary>
        /// Get Modified Accounts Accept dossier token, modified timestamp, and return modified/deleted Accounts.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>Task of SyncAccounts</returns>
        public async System.Threading.Tasks.Task<SyncAccounts> GetModifiedAccountsAsync (int? dossierId, string modifiedTimeStamp, string dossierToken)
        {
             ApiResponse<SyncAccounts> localVarResponse = await GetModifiedAccountsAsyncWithHttpInfo(dossierId, modifiedTimeStamp, dossierToken);
             return localVarResponse.Data;

        }

        /// <summary>
        /// Get Modified Accounts Accept dossier token, modified timestamp, and return modified/deleted Accounts.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="dossierId">Dossier ID</param>
        /// <param name="modifiedTimeStamp">Modified Timestamp</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <returns>Task of ApiResponse (SyncAccounts)</returns>
        public async System.Threading.Tasks.Task<ApiResponse<SyncAccounts>> GetModifiedAccountsAsyncWithHttpInfo (int? dossierId, string modifiedTimeStamp, string dossierToken)
        {
            // verify the required parameter 'dossierId' is set
            if (dossierId == null)
                throw new ApiException(400, "Missing required parameter 'dossierId' when calling AccountsApi->GetModifiedAccounts");
            // verify the required parameter 'modifiedTimeStamp' is set
            if (modifiedTimeStamp == null)
                throw new ApiException(400, "Missing required parameter 'modifiedTimeStamp' when calling AccountsApi->GetModifiedAccounts");
            // verify the required parameter 'dossierToken' is set
            if (dossierToken == null)
                throw new ApiException(400, "Missing required parameter 'dossierToken' when calling AccountsApi->GetModifiedAccounts");

            var localVarPath = "/dossiers/{dossierId}/accounts/modified";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (dossierId != null) localVarPathParams.Add("dossierId", this.Configuration.ApiClient.ParameterToString(dossierId)); // path parameter
            if (modifiedTimeStamp != null) localVarQueryParams.AddRange(this.Configuration.ApiClient.ParameterToKeyValuePairs("", "modifiedTimeStamp", modifiedTimeStamp)); // query parameter
            if (dossierToken != null) localVarHeaderParams.Add("dossierToken", this.Configuration.ApiClient.ParameterToString(dossierToken)); // header parameter

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await this.Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.GET, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("GetModifiedAccounts", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<SyncAccounts>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                (SyncAccounts) this.Configuration.ApiClient.Deserialize(localVarResponse, typeof(SyncAccounts)));
        }

        /// <summary>
        /// Modify or Create Account Accept dossier token and modified or new account.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Account</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <returns></returns>
        public void InsertUpdateAccount (AccountServiceData body, string dossierToken, int? dossierId)
        {
             InsertUpdateAccountWithHttpInfo(body, dossierToken, dossierId);
        }

        /// <summary>
        /// Modify or Create Account Accept dossier token and modified or new account.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Account</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <returns>ApiResponse of Object(void)</returns>
        public ApiResponse<Object> InsertUpdateAccountWithHttpInfo (AccountServiceData body, string dossierToken, int? dossierId)
        {
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling AccountsApi->InsertUpdateAccount");
            // verify the required parameter 'dossierToken' is set
            if (dossierToken == null)
                throw new ApiException(400, "Missing required parameter 'dossierToken' when calling AccountsApi->InsertUpdateAccount");
            // verify the required parameter 'dossierId' is set
            if (dossierId == null)
                throw new ApiException(400, "Missing required parameter 'dossierId' when calling AccountsApi->InsertUpdateAccount");

            var localVarPath = "/dossiers/{dossierId}/accounts";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (dossierId != null) localVarPathParams.Add("dossierId", this.Configuration.ApiClient.ParameterToString(dossierId)); // path parameter
            if (dossierToken != null) localVarHeaderParams.Add("dossierToken", this.Configuration.ApiClient.ParameterToString(dossierToken)); // header parameter
            if (body != null && body.GetType() != typeof(byte[]))
            {
                localVarPostBody = this.Configuration.ApiClient.Serialize(body); // http body (model) parameter
            }
            else
            {
                localVarPostBody = body; // byte array
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) this.Configuration.ApiClient.CallApi(localVarPath,
                Method.PUT, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("InsertUpdateAccount", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                null);
        }

        /// <summary>
        /// Modify or Create Account Accept dossier token and modified or new account.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Account</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <returns>Task of void</returns>
        public async System.Threading.Tasks.Task InsertUpdateAccountAsync (AccountServiceData body, string dossierToken, int? dossierId)
        {
             await InsertUpdateAccountAsyncWithHttpInfo(body, dossierToken, dossierId);

        }

        /// <summary>
        /// Modify or Create Account Accept dossier token and modified or new account.  
        /// </summary>
        /// <exception cref="IO.Swagger.Client.ApiException">Thrown when fails to make API call</exception>
        /// <param name="body">Account</param>
        /// <param name="dossierToken">Dossier Token</param>
        /// <param name="dossierId">Dossier ID</param>
        /// <returns>Task of ApiResponse</returns>
        public async System.Threading.Tasks.Task<ApiResponse<Object>> InsertUpdateAccountAsyncWithHttpInfo (AccountServiceData body, string dossierToken, int? dossierId)
        {
            // verify the required parameter 'body' is set
            if (body == null)
                throw new ApiException(400, "Missing required parameter 'body' when calling AccountsApi->InsertUpdateAccount");
            // verify the required parameter 'dossierToken' is set
            if (dossierToken == null)
                throw new ApiException(400, "Missing required parameter 'dossierToken' when calling AccountsApi->InsertUpdateAccount");
            // verify the required parameter 'dossierId' is set
            if (dossierId == null)
                throw new ApiException(400, "Missing required parameter 'dossierId' when calling AccountsApi->InsertUpdateAccount");

            var localVarPath = "/dossiers/{dossierId}/accounts";
            var localVarPathParams = new Dictionary<String, String>();
            var localVarQueryParams = new List<KeyValuePair<String, String>>();
            var localVarHeaderParams = new Dictionary<String, String>(this.Configuration.DefaultHeader);
            var localVarFormParams = new Dictionary<String, String>();
            var localVarFileParams = new Dictionary<String, FileParameter>();
            Object localVarPostBody = null;

            // to determine the Content-Type header
            String[] localVarHttpContentTypes = new String[] {
                "application/json"
            };
            String localVarHttpContentType = this.Configuration.ApiClient.SelectHeaderContentType(localVarHttpContentTypes);

            // to determine the Accept header
            String[] localVarHttpHeaderAccepts = new String[] {
                "application/json"
            };
            String localVarHttpHeaderAccept = this.Configuration.ApiClient.SelectHeaderAccept(localVarHttpHeaderAccepts);
            if (localVarHttpHeaderAccept != null)
                localVarHeaderParams.Add("Accept", localVarHttpHeaderAccept);

            if (dossierId != null) localVarPathParams.Add("dossierId", this.Configuration.ApiClient.ParameterToString(dossierId)); // path parameter
            if (dossierToken != null) localVarHeaderParams.Add("dossierToken", this.Configuration.ApiClient.ParameterToString(dossierToken)); // header parameter
            if (body != null && body.GetType() != typeof(byte[]))
            {
                localVarPostBody = this.Configuration.ApiClient.Serialize(body); // http body (model) parameter
            }
            else
            {
                localVarPostBody = body; // byte array
            }

            // make the HTTP request
            IRestResponse localVarResponse = (IRestResponse) await this.Configuration.ApiClient.CallApiAsync(localVarPath,
                Method.PUT, localVarQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarFileParams,
                localVarPathParams, localVarHttpContentType);

            int localVarStatusCode = (int) localVarResponse.StatusCode;

            if (ExceptionFactory != null)
            {
                Exception exception = ExceptionFactory("InsertUpdateAccount", localVarResponse);
                if (exception != null) throw exception;
            }

            return new ApiResponse<Object>(localVarStatusCode,
                localVarResponse.Headers.ToDictionary(x => x.Name, x => string.Join(",", x.Value)),
                null);
        }

    }
}
