/* 
 * Octopus Rest API
 *
 * API to connect with Octopus
 *
 * OpenAPI spec version: 1.0
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 */
using System;
using System.Linq;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using System.ComponentModel.DataAnnotations;
using SwaggerDateConverter = IO.Swagger.Client.SwaggerDateConverter;

namespace IO.Swagger.Model
{
    /// <summary>
    /// Intrastat Data
    /// </summary>
    [DataContract]
        public partial class IntrastatServiceData :  IEquatable<IntrastatServiceData>, IValidatableObject
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="IntrastatServiceData" /> class.
        /// </summary>
        /// <param name="isoCountrycode">ISO code member state of origin/destination, eg. \&quot;BE\&quot;.</param>
        /// <param name="transactionCode">Transaction code :&lt;ul&gt;&lt;li&gt;1 : Transactions accompanied by actual or planned transfer of ownership from residents to non-residents with a financial&lt;br/&gt;or other compensation(except the transactions listed under 2, 7, and 8).&lt;/li&gt;&lt;li&gt;2 : Return shipping and free replacement of goods after registration of the original transaction.&lt;/li&gt;&lt;li&gt;3 : Transactions involving ownership transfer without financial or \&quot;in kind\&quot; compensations (ex. shipment of relief goods).&lt;/li&gt;&lt;li&gt;4 : Transactions with a view to processing under contract ( without ownership transfer to the processor).&lt;/li&gt;&lt;li&gt;5 : Transactions after subcontracting (without property tranfer to the subcontractant).&lt;/li&gt;&lt;li&gt;6 : Transactions after repair or maintenance, except in case of intergovernmental programs for coordinated manufacturing.&lt;br/&gt;This service can be free or paid.  This transactions include no (future) tranfer of property rights.&lt;/li&gt;&lt;li&gt;7 : Transactions in relation to common defense programs or other intergovernmental programs for common manufacturing.&lt;/li&gt;&lt;li&gt;8 : Transactions within the framework of a contract for general construction work or road building or hydraulic engineering&lt;br/&gt; where the goods are to be invoiced separately and an invoice for the entire contract is drawn up.&lt;/li&gt;&lt;li&gt;9 : Other transactions that do not match with existing codes.&lt;/li&gt;&lt;/ul&gt;.</param>
        /// <param name="productCode">Product Code.</param>
        /// <param name="region">Region of origin/destination : &lt;ul&gt;&lt;li&gt;1 : Flemish region&lt;li&gt;2 : Walloon region&lt;li&gt;3 : Brussels capital region&lt;/ul&gt;.</param>
        /// <param name="weight">Net weight.</param>
        /// <param name="unitCount">Number of units.</param>
        /// <param name="transportCode">Transport code : &lt;ul&gt;&lt;li&gt;0 : No entry&lt;li&gt;1 : Transport by sea (also if wagon, motorvehicle, trailer, trailer without tractor or lighter aboard of ship)&lt;li&gt;2 : Transport by rail (even Truck on wagon)&lt;li&gt;3 : Road transport&lt;li&gt;4 : Air transport&lt;li&gt;5 : Send by mail&lt;li&gt;7 : Fixed transport installation(eg. pipe-line, high voltage line)&lt;li&gt;8 : Inland waterway transport&lt;li&gt;9 : On his own (applies to sold goods that cross the border on their own when exported or imported &lt;br/&gt; such as airplanes, trucks, schips, etc...)&lt;/ul&gt;.</param>
        /// <param name="incoTerms">Inco Terms : &lt;ul&gt;&lt;li&gt;CFR - Cost and Freight&lt;li&gt;CIF - Cost, Insurance and Freight&lt;li&gt;CIP - Free transport and insurance up to a given destination&lt;li&gt;CPT - Transport free up to a given destination&lt;li&gt;DAP - Delivered At Place EXW&lt;li&gt;DAT - Franco at terminal&lt;li&gt;DDP - Delivered Duty Paid&lt;li&gt;EXW - Ex works&lt;li&gt;FAS - Franco alongside ship&lt;li&gt;FCA - Free Carrier&lt;li&gt;FOB - Free On Board&lt;li&gt;N   - No entry&lt;/ul&gt;.</param>
        /// <param name="originCountry">Country of origin code.</param>
        public IntrastatServiceData(string isoCountrycode = default(string), int? transactionCode = default(int?), string productCode = default(string), int? region = default(int?), double? weight = default(double?), double? unitCount = default(double?), int? transportCode = default(int?), string incoTerms = default(string), string originCountry = default(string))
        {
            this.IsoCountrycode = isoCountrycode;
            this.TransactionCode = transactionCode;
            this.ProductCode = productCode;
            this.Region = region;
            this.Weight = weight;
            this.UnitCount = unitCount;
            this.TransportCode = transportCode;
            this.IncoTerms = incoTerms;
            this.OriginCountry = originCountry;
        }
        
        /// <summary>
        /// ISO code member state of origin/destination, eg. \&quot;BE\&quot;
        /// </summary>
        /// <value>ISO code member state of origin/destination, eg. \&quot;BE\&quot;</value>
        [DataMember(Name="isoCountrycode", EmitDefaultValue=false)]
        public string IsoCountrycode { get; set; }

        /// <summary>
        /// Transaction code :&lt;ul&gt;&lt;li&gt;1 : Transactions accompanied by actual or planned transfer of ownership from residents to non-residents with a financial&lt;br/&gt;or other compensation(except the transactions listed under 2, 7, and 8).&lt;/li&gt;&lt;li&gt;2 : Return shipping and free replacement of goods after registration of the original transaction.&lt;/li&gt;&lt;li&gt;3 : Transactions involving ownership transfer without financial or \&quot;in kind\&quot; compensations (ex. shipment of relief goods).&lt;/li&gt;&lt;li&gt;4 : Transactions with a view to processing under contract ( without ownership transfer to the processor).&lt;/li&gt;&lt;li&gt;5 : Transactions after subcontracting (without property tranfer to the subcontractant).&lt;/li&gt;&lt;li&gt;6 : Transactions after repair or maintenance, except in case of intergovernmental programs for coordinated manufacturing.&lt;br/&gt;This service can be free or paid.  This transactions include no (future) tranfer of property rights.&lt;/li&gt;&lt;li&gt;7 : Transactions in relation to common defense programs or other intergovernmental programs for common manufacturing.&lt;/li&gt;&lt;li&gt;8 : Transactions within the framework of a contract for general construction work or road building or hydraulic engineering&lt;br/&gt; where the goods are to be invoiced separately and an invoice for the entire contract is drawn up.&lt;/li&gt;&lt;li&gt;9 : Other transactions that do not match with existing codes.&lt;/li&gt;&lt;/ul&gt;
        /// </summary>
        /// <value>Transaction code :&lt;ul&gt;&lt;li&gt;1 : Transactions accompanied by actual or planned transfer of ownership from residents to non-residents with a financial&lt;br/&gt;or other compensation(except the transactions listed under 2, 7, and 8).&lt;/li&gt;&lt;li&gt;2 : Return shipping and free replacement of goods after registration of the original transaction.&lt;/li&gt;&lt;li&gt;3 : Transactions involving ownership transfer without financial or \&quot;in kind\&quot; compensations (ex. shipment of relief goods).&lt;/li&gt;&lt;li&gt;4 : Transactions with a view to processing under contract ( without ownership transfer to the processor).&lt;/li&gt;&lt;li&gt;5 : Transactions after subcontracting (without property tranfer to the subcontractant).&lt;/li&gt;&lt;li&gt;6 : Transactions after repair or maintenance, except in case of intergovernmental programs for coordinated manufacturing.&lt;br/&gt;This service can be free or paid.  This transactions include no (future) tranfer of property rights.&lt;/li&gt;&lt;li&gt;7 : Transactions in relation to common defense programs or other intergovernmental programs for common manufacturing.&lt;/li&gt;&lt;li&gt;8 : Transactions within the framework of a contract for general construction work or road building or hydraulic engineering&lt;br/&gt; where the goods are to be invoiced separately and an invoice for the entire contract is drawn up.&lt;/li&gt;&lt;li&gt;9 : Other transactions that do not match with existing codes.&lt;/li&gt;&lt;/ul&gt;</value>
        [DataMember(Name="transactionCode", EmitDefaultValue=false)]
        public int? TransactionCode { get; set; }

        /// <summary>
        /// Product Code
        /// </summary>
        /// <value>Product Code</value>
        [DataMember(Name="productCode", EmitDefaultValue=false)]
        public string ProductCode { get; set; }

        /// <summary>
        /// Region of origin/destination : &lt;ul&gt;&lt;li&gt;1 : Flemish region&lt;li&gt;2 : Walloon region&lt;li&gt;3 : Brussels capital region&lt;/ul&gt;
        /// </summary>
        /// <value>Region of origin/destination : &lt;ul&gt;&lt;li&gt;1 : Flemish region&lt;li&gt;2 : Walloon region&lt;li&gt;3 : Brussels capital region&lt;/ul&gt;</value>
        [DataMember(Name="region", EmitDefaultValue=false)]
        public int? Region { get; set; }

        /// <summary>
        /// Net weight
        /// </summary>
        /// <value>Net weight</value>
        [DataMember(Name="weight", EmitDefaultValue=false)]
        public double? Weight { get; set; }

        /// <summary>
        /// Number of units
        /// </summary>
        /// <value>Number of units</value>
        [DataMember(Name="unitCount", EmitDefaultValue=false)]
        public double? UnitCount { get; set; }

        /// <summary>
        /// Transport code : &lt;ul&gt;&lt;li&gt;0 : No entry&lt;li&gt;1 : Transport by sea (also if wagon, motorvehicle, trailer, trailer without tractor or lighter aboard of ship)&lt;li&gt;2 : Transport by rail (even Truck on wagon)&lt;li&gt;3 : Road transport&lt;li&gt;4 : Air transport&lt;li&gt;5 : Send by mail&lt;li&gt;7 : Fixed transport installation(eg. pipe-line, high voltage line)&lt;li&gt;8 : Inland waterway transport&lt;li&gt;9 : On his own (applies to sold goods that cross the border on their own when exported or imported &lt;br/&gt; such as airplanes, trucks, schips, etc...)&lt;/ul&gt;
        /// </summary>
        /// <value>Transport code : &lt;ul&gt;&lt;li&gt;0 : No entry&lt;li&gt;1 : Transport by sea (also if wagon, motorvehicle, trailer, trailer without tractor or lighter aboard of ship)&lt;li&gt;2 : Transport by rail (even Truck on wagon)&lt;li&gt;3 : Road transport&lt;li&gt;4 : Air transport&lt;li&gt;5 : Send by mail&lt;li&gt;7 : Fixed transport installation(eg. pipe-line, high voltage line)&lt;li&gt;8 : Inland waterway transport&lt;li&gt;9 : On his own (applies to sold goods that cross the border on their own when exported or imported &lt;br/&gt; such as airplanes, trucks, schips, etc...)&lt;/ul&gt;</value>
        [DataMember(Name="transportCode", EmitDefaultValue=false)]
        public int? TransportCode { get; set; }

        /// <summary>
        /// Inco Terms : &lt;ul&gt;&lt;li&gt;CFR - Cost and Freight&lt;li&gt;CIF - Cost, Insurance and Freight&lt;li&gt;CIP - Free transport and insurance up to a given destination&lt;li&gt;CPT - Transport free up to a given destination&lt;li&gt;DAP - Delivered At Place EXW&lt;li&gt;DAT - Franco at terminal&lt;li&gt;DDP - Delivered Duty Paid&lt;li&gt;EXW - Ex works&lt;li&gt;FAS - Franco alongside ship&lt;li&gt;FCA - Free Carrier&lt;li&gt;FOB - Free On Board&lt;li&gt;N   - No entry&lt;/ul&gt;
        /// </summary>
        /// <value>Inco Terms : &lt;ul&gt;&lt;li&gt;CFR - Cost and Freight&lt;li&gt;CIF - Cost, Insurance and Freight&lt;li&gt;CIP - Free transport and insurance up to a given destination&lt;li&gt;CPT - Transport free up to a given destination&lt;li&gt;DAP - Delivered At Place EXW&lt;li&gt;DAT - Franco at terminal&lt;li&gt;DDP - Delivered Duty Paid&lt;li&gt;EXW - Ex works&lt;li&gt;FAS - Franco alongside ship&lt;li&gt;FCA - Free Carrier&lt;li&gt;FOB - Free On Board&lt;li&gt;N   - No entry&lt;/ul&gt;</value>
        [DataMember(Name="incoTerms", EmitDefaultValue=false)]
        public string IncoTerms { get; set; }

        /// <summary>
        /// Country of origin code
        /// </summary>
        /// <value>Country of origin code</value>
        [DataMember(Name="originCountry", EmitDefaultValue=false)]
        public string OriginCountry { get; set; }

        /// <summary>
        /// Returns the string presentation of the object
        /// </summary>
        /// <returns>String presentation of the object</returns>
        public override string ToString()
        {
            var sb = new StringBuilder();
            sb.Append("class IntrastatServiceData {\n");
            sb.Append("  IsoCountrycode: ").Append(IsoCountrycode).Append("\n");
            sb.Append("  TransactionCode: ").Append(TransactionCode).Append("\n");
            sb.Append("  ProductCode: ").Append(ProductCode).Append("\n");
            sb.Append("  Region: ").Append(Region).Append("\n");
            sb.Append("  Weight: ").Append(Weight).Append("\n");
            sb.Append("  UnitCount: ").Append(UnitCount).Append("\n");
            sb.Append("  TransportCode: ").Append(TransportCode).Append("\n");
            sb.Append("  IncoTerms: ").Append(IncoTerms).Append("\n");
            sb.Append("  OriginCountry: ").Append(OriginCountry).Append("\n");
            sb.Append("}\n");
            return sb.ToString();
        }
  
        /// <summary>
        /// Returns the JSON string presentation of the object
        /// </summary>
        /// <returns>JSON string presentation of the object</returns>
        public virtual string ToJson()
        {
            return JsonConvert.SerializeObject(this, Formatting.Indented);
        }

        /// <summary>
        /// Returns true if objects are equal
        /// </summary>
        /// <param name="input">Object to be compared</param>
        /// <returns>Boolean</returns>
        public override bool Equals(object input)
        {
            return this.Equals(input as IntrastatServiceData);
        }

        /// <summary>
        /// Returns true if IntrastatServiceData instances are equal
        /// </summary>
        /// <param name="input">Instance of IntrastatServiceData to be compared</param>
        /// <returns>Boolean</returns>
        public bool Equals(IntrastatServiceData input)
        {
            if (input == null)
                return false;

            return 
                (
                    this.IsoCountrycode == input.IsoCountrycode ||
                    (this.IsoCountrycode != null &&
                    this.IsoCountrycode.Equals(input.IsoCountrycode))
                ) && 
                (
                    this.TransactionCode == input.TransactionCode ||
                    (this.TransactionCode != null &&
                    this.TransactionCode.Equals(input.TransactionCode))
                ) && 
                (
                    this.ProductCode == input.ProductCode ||
                    (this.ProductCode != null &&
                    this.ProductCode.Equals(input.ProductCode))
                ) && 
                (
                    this.Region == input.Region ||
                    (this.Region != null &&
                    this.Region.Equals(input.Region))
                ) && 
                (
                    this.Weight == input.Weight ||
                    (this.Weight != null &&
                    this.Weight.Equals(input.Weight))
                ) && 
                (
                    this.UnitCount == input.UnitCount ||
                    (this.UnitCount != null &&
                    this.UnitCount.Equals(input.UnitCount))
                ) && 
                (
                    this.TransportCode == input.TransportCode ||
                    (this.TransportCode != null &&
                    this.TransportCode.Equals(input.TransportCode))
                ) && 
                (
                    this.IncoTerms == input.IncoTerms ||
                    (this.IncoTerms != null &&
                    this.IncoTerms.Equals(input.IncoTerms))
                ) && 
                (
                    this.OriginCountry == input.OriginCountry ||
                    (this.OriginCountry != null &&
                    this.OriginCountry.Equals(input.OriginCountry))
                );
        }

        /// <summary>
        /// Gets the hash code
        /// </summary>
        /// <returns>Hash code</returns>
        public override int GetHashCode()
        {
            unchecked // Overflow is fine, just wrap
            {
                int hashCode = 41;
                if (this.IsoCountrycode != null)
                    hashCode = hashCode * 59 + this.IsoCountrycode.GetHashCode();
                if (this.TransactionCode != null)
                    hashCode = hashCode * 59 + this.TransactionCode.GetHashCode();
                if (this.ProductCode != null)
                    hashCode = hashCode * 59 + this.ProductCode.GetHashCode();
                if (this.Region != null)
                    hashCode = hashCode * 59 + this.Region.GetHashCode();
                if (this.Weight != null)
                    hashCode = hashCode * 59 + this.Weight.GetHashCode();
                if (this.UnitCount != null)
                    hashCode = hashCode * 59 + this.UnitCount.GetHashCode();
                if (this.TransportCode != null)
                    hashCode = hashCode * 59 + this.TransportCode.GetHashCode();
                if (this.IncoTerms != null)
                    hashCode = hashCode * 59 + this.IncoTerms.GetHashCode();
                if (this.OriginCountry != null)
                    hashCode = hashCode * 59 + this.OriginCountry.GetHashCode();
                return hashCode;
            }
        }

        /// <summary>
        /// To validate all properties of the instance
        /// </summary>
        /// <param name="validationContext">Validation context</param>
        /// <returns>Validation Result</returns>
        IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> IValidatableObject.Validate(ValidationContext validationContext)
        {
            yield break;
        }
    }
}
